#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Materials" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

import sys

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *


RADIUS = 4.


class myGLCanvas(object):
    def __init__(self):
        self.highShininess = False
        self.lowSpecularity = False
        self.emission = False
        self._angle = -60

        glutInit(sys.argv)
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
        glutInitWindowSize(400, 400)
        glutCreateWindow(b"Materials - videotutorialsrock.com")

        self.initRendering()
              
        glutDisplayFunc(self.OnPaint)
        glutKeyboardFunc(self.handleKeypress)
        glutReshapeFunc(self.OnResize)

        glutTimerFunc(25, self.update, 0)
        
        glutMainLoop()
        return 0
        
    
    def initRendering(self):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_NORMALIZE)
        glShadeModel(GL_SMOOTH)
        glDisable(GL_COLOR_MATERIAL)


    def OnPaint(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
                
        glTranslatef(0.0, 0.0, -20.0)
        glRotatef(self._angle, 0, 1, 0)
        
        ambientLight = [0.2, 0.2, 0.2, 1.0]
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight)
        
        lightColor = [0.6, 0.6, 0.6, 1.0]
        lightPos = [1.5 * RADIUS, 2 * RADIUS, 1.5 * RADIUS, 1.0]
        glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor)
        glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor)
        glLightfv(GL_LIGHT0, GL_POSITION, lightPos)
        
        if self.lowSpecularity : specularity = 0.3
        else : specularity = 1
	
        if self.emission : emissivity = 0.05
        else : emissivity = 0
	
        if self.highShininess : shininess = 25
        else : shininess = 12
        
        materialColor = [0.2, 0., 1.0, 1.0]
        materialSpecular = [specularity, specularity, specularity, 1.0]
        materialEmission = [emissivity, emissivity, emissivity, 1.0]
        
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, materialColor)
        glMaterialfv(GL_FRONT, GL_SPECULAR, materialSpecular)
        glMaterialfv(GL_FRONT, GL_EMISSION, materialEmission)
        glMaterialf(GL_FRONT, GL_SHININESS, shininess)
        
        glutSolidSphere(RADIUS, 150, 80)

        glutSwapBuffers()


    def OnResize(self, w, h):
        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)


    def handleKeypress(self, key, x, y):
        if ord(key) == 27 : exit()
        elif ord(key) == 115 or key == 83 :
            self.highShininess = not self.highShininess
        elif ord(key) == 112 or key == 80 :
            self.lowSpecularity = not self.lowSpecularity
        elif ord(key) == 101 or key == 69 :
            self.emission = not self.emission


    def update(self, value):
        self._angle += 2.
        if self._angle > 360. : self._angle -= 360.
        
        glutPostRedisplay()
        glutTimerFunc(25, self.update, 0)
        
        
if __name__ == '__main__' :
    canvas = myGLCanvas()
